import logging
from typing import Optional
from linkwiz.config import config
import fnmatch


def get_browser_for_url(hostname) -> Optional[str]:
    try:
        for pattern, browser in config.rules_fnmatch.items():
            if fnmatch.fnmatch(hostname, pattern):
                logging.info(f"Matched {hostname} to {browser}")
                return browser
        return config.rules_hostname.get(hostname, None)
    except Exception as e:
        logging.warning(f"Error matching {hostname} to {pattern}: {e}")


def find_matching_browser(browsers, url, hostname):
    browser = get_browser_for_url(hostname)
    if browser is None:
        logging.info(f"No match for {url}")
        return
    for name, path in browsers.items():
        if browser == name:
            logging.info(f"Opening {url} with {name}")
            return path, url



import os
from typing import Dict
import tomllib
from xdg import BaseDirectory
import tomli_w


class Config:
    def __init__(self, config_path: str = None):
        self.config_path = config_path or os.path.join(
            BaseDirectory.xdg_config_home, "linkwiz", "linkwiz.toml"
        )
        self._config = self._load_config()

    def _load_config(self) -> Dict:
        """
        Load the configuration file, creating a new one with defaults if it doesn't exist.
        """
        config_dir = os.path.dirname(self.config_path)
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)

        if os.path.exists(self.config_path):
            with open(self.config_path, "rb") as f:
                return tomllib.load(f)
        else:
            config = {
                "main": {"auto_find_browsers": True},
                "browsers": {},
                "rules": {"fnmatch": {}, "hostname": {}},
                "features": {"remove_track": False},
            }
            with open(self.config_path, "wb") as f:
                tomli_w.dump(config, f)
            return config

    def save_config(self):
        """
        Save the configuration file.
        """
        with open(self.config_path, "wb") as f:
            tomli_w.dump(self._config, f)

    def add_rules(self, hostname: str, browser_name: str):
        """
        Add rules to the configuration file.
        """
        if "rules" not in self._config:
            self._config["rules"] = {"regex": {}, "fnmatch": {}, "hostname": {}}
        rules = self._config["rules"]
        if "hostname" not in rules:
            rules["hostname"] = {}
        rules["hostname"][hostname] = browser_name
        self.save_config()

    @property
    def main(self) -> Dict:
        """
        Get the main section from the configuration.
        """
        return self._config.get("main", {})

    @property
    def browsers(self) -> Dict:
        """
        Get the browsers from the configuration.
        """
        return self._config.get("browsers", {})

    @property
    def rules(self) -> Dict:
        """
        Get the rules from the configuration.
        """
        return self._config.get("rules", {})

    @property
    def rules_fnmatch(self) -> Dict:
        """
        Get the fnmatch rules from the configuration.
        """
        return self.rules.get("fnmatch", {})

    @property
    def rules_hostname(self) -> Dict:
        """
        Get the hostname rules from the configuration.
        """
        return self.rules.get("hostname", {})

    @property
    def features(self) -> Dict:
        """
        Get the features from the configuration.
        """
        return self._config.get("features", {})


config = Config()



from urllib.parse import urlparse
import logging

from linkwiz.browser import get_browsers
from linkwiz.match import find_matching_browser
from linkwiz.gui import LinkwizGUI
from linkwiz.launch import launch_browser
from linkwiz.config import config


def process_url(url):
    """Process the provided URL."""
    parsed_url = urlparse(url)

    if parsed_url.scheme not in ("http", "https"):
        raise ValueError("Invalid URL.")

    if config.features.get("remove_track", False):
        url = remove_tracking(url)
    browsers = get_browsers()

    try:
        launch_browser_command = find_matching_browser(
            browsers, parsed_url.geturl(), parsed_url.netloc
        )
        if launch_browser_command:
            launch_browser(*launch_browser_command)
            return

        app = LinkwizGUI(browsers, url)
        launch_browser_command = app.run()
        launch_browser(*launch_browser_command)

    except Exception as e:
        logging.error(f"Error processing URL: {e}")


def remove_tracking(url):
    """Remove tracking parameters from the URL."""
    try:
        import unalix
        return unalix.clear_url(url=url)
    except ImportError:
        logging.warning("The 'unalix' package is not installed. Cannot remove tracking parameters.")
        return url


import logging
from pathlib import Path
import subprocess
from typing import Dict, List
from xdg import DesktopEntry
from linkwiz.config import config

SELF_DESKTOP: str = "linkwiz.desktop"
HTTP_HANDLER: str = "x-scheme-handler/http"

DESKTOP_PATHS = [
    Path("/usr/share/applications/"),
    Path.home() / ".local/share/applications/",
]

MIMEINFO_PATHS = [
    Path("/usr/share/applications/mimeinfo.cache"),
    Path.home() / ".local/share/applications/mimeinfo.cache",
]


def get_browsers() -> Dict[str, Path]:
    """Get the name and exec path of browsers."""
    try:
        installed_browsers = []
        if config.main.get("auto_find_browsers", True):
            installed_browsers = find_installed_browsers()

        browsers = get_browser_exec(installed_browsers)
        browsers.update(config.browsers)

        return browsers
    except subprocess.CalledProcessError:
        logging.error("Error getting installed browsers")
        exit(1)


def find_installed_browsers() -> List[str]:
    """Get the name of installed browsers."""
    installed_browsers = set()
    for path in MIMEINFO_PATHS:
        if not path.exists():
            continue
        with open(path, "r") as f:
            for line in f:
                if not line.startswith(HTTP_HANDLER):
                    continue
                browsers = line.split("=")[-1].strip().split(";")
                installed_browsers.update(browsers)
                break
    installed_browsers.discard(SELF_DESKTOP)
    return list(installed_browsers)


def get_browser_exec(browsers_desktop: List[str]) -> Dict[str, Path]:
    """Get the exec path of installed browsers."""
    browsers_exec: Dict[str, Path] = {}
    for path in DESKTOP_PATHS:
        if not path.exists():
            continue
        for entry in path.glob("*.desktop"):
            if entry.name not in browsers_desktop:
                continue
            desktop_entry = DesktopEntry.DesktopEntry(str(entry))
            name: str = desktop_entry.getName()
            execpath: str = desktop_entry.getExec()
            browsers_exec[name] = Path(execpath)
    return browsers_exec
